@@ -1,28 +1,24 @@
 import binascii
+import json
 import os
 import re
 import time
 import urllib.parse
-import json
+import warnings
+from collections.abc import Callable
+from collections.abc import Iterable
+from collections.abc import Iterator
+from collections.abc import Mapping
+from collections.abc import Sequence
 from dataclasses import dataclass
 from dataclasses import fields
 from email.utils import formatdate
 from email.utils import mktime_tz
 from email.utils import parsedate_tz
-from typing import Callable
-from typing import Dict
-from typing import Iterable
-from typing import Iterator
-from typing import List
-from typing import Mapping
-from typing import Optional
-from typing import Tuple
-from typing import Union
-from typing import cast
 from typing import Any
+from typing import cast
 
 from mitmproxy import flow
-from mitmproxy.websocket import WebSocketData
 from mitmproxy.coretypes import multidict
 from mitmproxy.coretypes import serializable
 from mitmproxy.net import encoding
@@ -37,6 +33,7 @@ from mitmproxy.utils import strutils
 from mitmproxy.utils import typecheck
 from mitmproxy.utils.strutils import always_bytes
 from mitmproxy.utils.strutils import always_str
+from mitmproxy.websocket import WebSocketData
 
 
 # While headers _should_ be ASCII, it's not uncommon for certain headers to be utf-8 encoded.
@@ -44,7 +41,7 @@ def _native(x: bytes) -> str:
     return x.decode("utf-8", "surrogateescape")
 
 
-def _always_bytes(x: Union[str, bytes]) -> bytes:
+def _always_bytes(x: str | bytes) -> bytes:
     return strutils.always_bytes(x, "utf-8", "surrogateescape")
 
 
@@ -93,7 +90,7 @@ class Headers(multidict.MultiDict):  # type: ignore
      - For use with the "Set-Cookie" and "Cookie" headers, either use `Response.cookies` or see `Headers.get_all`.
     """
 
-    def __init__(self, fields: Iterable[Tuple[bytes, bytes]] = (), **headers):
+    def __init__(self, fields: Iterable[tuple[bytes, bytes]] = (), **headers):
         """
         *Args:*
          - *fields:* (optional) list of ``(name, value)`` header byte tuples,
@@ -112,12 +109,14 @@ class Headers(multidict.MultiDict):  # type: ignore
                 raise TypeError("Header fields must be bytes.")
 
         # content_type -> content-type
-        self.update({
-            _always_bytes(name).replace(b"_", b"-"): _always_bytes(value)
-            for name, value in headers.items()
-        })
+        self.update(
+            {
+                _always_bytes(name).replace(b"_", b"-"): _always_bytes(value)
+                for name, value in headers.items()
+            }
+        )
 
-    fields: Tuple[Tuple[bytes, bytes], ...]
+    fields: tuple[tuple[bytes, bytes], ...]
 
     @staticmethod
     def _reduce_values(values) -> str:
@@ -135,7 +134,7 @@ class Headers(multidict.MultiDict):  # type: ignore
         else:
             return b""
 
-    def __delitem__(self, key: Union[str, bytes]) -> None:
+    def __delitem__(self, key: str | bytes) -> None:
         key = _always_bytes(key)
         super().__delitem__(key)
 
@@ -143,7 +142,7 @@ class Headers(multidict.MultiDict):  # type: ignore
         for x in super().__iter__():
             yield _native(x)
 
-    def get_all(self, name: Union[str, bytes]) -> List[str]:
+    def get_all(self, name: str | bytes) -> list[str]:
         """
         Like `Headers.get`, but does not fold multiple headers into a single one.
         This is useful for Set-Cookie and Cookie headers, which do not support folding.
@@ -154,12 +153,9 @@ class Headers(multidict.MultiDict):  # type: ignore
          - <https://datatracker.ietf.org/doc/html/rfc7540#section-8.1.2.5>
         """
         name = _always_bytes(name)
-        return [
-            _native(x) for x in
-            super().get_all(name)
-        ]
+        return [_native(x) for x in super().get_all(name)]
 
-    def set_all(self, name: Union[str, bytes], values: List[Union[str, bytes]]):
+    def set_all(self, name: str | bytes, values: Iterable[str | bytes]):
         """
         Explicitly set multiple headers for the given key.
         See `Headers.get_all`.
@@ -168,17 +164,14 @@ class Headers(multidict.MultiDict):  # type: ignore
         values = [_always_bytes(x) for x in values]
         return super().set_all(name, values)
 
-    def insert(self, index: int, key: Union[str, bytes], value: Union[str, bytes]):
+    def insert(self, index: int, key: str | bytes, value: str | bytes):
         key = _always_bytes(key)
         value = _always_bytes(value)
         super().insert(index, key, value)
 
     def items(self, multi=False):
         if multi:
-            return (
-                (_native(k), _native(v))
-                for k, v in self.fields
-            )
+            return ((_native(k), _native(v)) for k, v in self.fields)
         else:
             return super().items()
 
@@ -187,13 +180,14 @@ class Headers(multidict.MultiDict):  # type: ignore
 class MessageData(serializable.Serializable):
     http_version: bytes
     headers: Headers
-    content: Optional[bytes]
-    trailers: Optional[Headers]
+    content: bytes | None
+    trailers: Headers | None
     timestamp_start: float
-    timestamp_end: Optional[float]
+    timestamp_end: float | None
 
     # noinspection PyUnreachableCode
     if __debug__:
+
         def __post_init__(self):
             for field in fields(self):
                 val = getattr(self, field.name)
@@ -250,7 +244,7 @@ class Message(serializable.Serializable):
         self.data.set_state(state)
 
     data: MessageData
-    stream: Union[Callable[[bytes], Union[Iterable[bytes], bytes]], bool] = False
+    stream: Callable[[bytes], Iterable[bytes] | bytes] | bool = False
     """
     This attribute controls if the message body should be streamed.
 
@@ -273,8 +267,10 @@ class Message(serializable.Serializable):
         return self.data.http_version.decode("utf-8", "surrogateescape")
 
     @http_version.setter
-    def http_version(self, http_version: Union[str, bytes]) -> None:
-        self.data.http_version = strutils.always_bytes(http_version, "utf-8", "surrogateescape")
+    def http_version(self, http_version: str | bytes) -> None:
+        self.data.http_version = strutils.always_bytes(
+            http_version, "utf-8", "surrogateescape"
+        )
 
     @property
     def is_http10(self) -> bool:
@@ -288,6 +284,10 @@ class Message(serializable.Serializable):
     def is_http2(self) -> bool:
         return self.data.http_version == b"HTTP/2.0"
 
+    @property
+    def is_http3(self) -> bool:
+        return self.data.http_version == b"HTTP/3"
+
     @property
     def headers(self) -> Headers:
         """
@@ -300,18 +300,18 @@ class Message(serializable.Serializable):
         self.data.headers = h
 
     @property
-    def trailers(self) -> Optional[Headers]:
+    def trailers(self) -> Headers | None:
         """
         The [HTTP trailers](https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Trailer).
         """
         return self.data.trailers
 
     @trailers.setter
-    def trailers(self, h: Optional[Headers]) -> None:
+    def trailers(self, h: Headers | None) -> None:
         self.data.trailers = h
 
     @property
-    def raw_content(self) -> Optional[bytes]:
+    def raw_content(self) -> bytes | None:
         """
         The raw (potentially compressed) HTTP message body.
 
@@ -322,11 +322,11 @@ class Message(serializable.Serializable):
         return self.data.content
 
     @raw_content.setter
-    def raw_content(self, content: Optional[bytes]) -> None:
+    def raw_content(self, content: bytes | None) -> None:
         self.data.content = content
 
     @property
-    def content(self) -> Optional[bytes]:
+    def content(self) -> bytes | None:
         """
         The uncompressed HTTP message body as bytes.
 
@@ -337,11 +337,11 @@ class Message(serializable.Serializable):
         return self.get_content()
 
     @content.setter
-    def content(self, value: Optional[bytes]) -> None:
+    def content(self, value: bytes | None) -> None:
         self.set_content(value)
 
     @property
-    def text(self) -> Optional[str]:
+    def text(self) -> str | None:
         """
         The uncompressed and decoded HTTP message body as text.
 
@@ -352,10 +352,10 @@ class Message(serializable.Serializable):
         return self.get_text()
 
     @text.setter
-    def text(self, value: Optional[str]) -> None:
+    def text(self, value: str | None) -> None:
         self.set_text(value)
 
-    def set_content(self, value: Optional[bytes]) -> None:
+    def set_content(self, value: bytes | None) -> None:
         if value is None:
             self.raw_content = None
             return
@@ -380,7 +380,7 @@ class Message(serializable.Serializable):
         else:
             self.headers["content-length"] = str(len(self.raw_content))
 
-    def get_content(self, strict: bool = True) -> Optional[bytes]:
+    def get_content(self, strict: bool = True) -> bytes | None:
         """
         Similar to `Message.content`, but does not raise if `strict` is `False`.
         Instead, the compressed message body is returned as-is.
@@ -402,7 +402,7 @@ class Message(serializable.Serializable):
         else:
             return self.raw_content
 
-    def _get_content_type_charset(self) -> Optional[str]:
+    def _get_content_type_charset(self) -> str | None:
         ct = parse_content_type(self.headers.get("content-type", ""))
         if ct:
             return ct[2].get("charset")
@@ -415,13 +415,17 @@ class Message(serializable.Serializable):
                 enc = "utf8"
         if not enc:
             if "html" in self.headers.get("content-type", ""):
-                meta_charset = re.search(rb"""<meta[^>]+charset=['"]?([^'">]+)""", content, re.IGNORECASE)
+                meta_charset = re.search(
+                    rb"""<meta[^>]+charset=['"]?([^'">]+)""", content, re.IGNORECASE
+                )
                 if meta_charset:
                     enc = meta_charset.group(1).decode("ascii", "ignore")
         if not enc:
             if "text/css" in self.headers.get("content-type", ""):
                 # @charset rule must be the very first thing.
-                css_charset = re.match(rb"""@charset "([^"]+)";""", content, re.IGNORECASE)
+                css_charset = re.match(
+                    rb"""@charset "([^"]+)";""", content, re.IGNORECASE
+                )
                 if css_charset:
                     enc = css_charset.group(1).decode("ascii", "ignore")
         if not enc:
@@ -432,7 +436,7 @@ class Message(serializable.Serializable):
 
         return enc
 
-    def set_text(self, text: Optional[str]) -> None:
+    def set_text(self, text: str | None) -> None:
         if text is None:
             self.content = None
             return
@@ -442,13 +446,17 @@ class Message(serializable.Serializable):
             self.content = cast(bytes, encoding.encode(text, enc))
         except ValueError:
             # Fall back to UTF-8 and update the content-type header.
-            ct = parse_content_type(self.headers.get("content-type", "")) or ("text", "plain", {})
+            ct = parse_content_type(self.headers.get("content-type", "")) or (
+                "text",
+                "plain",
+                {},
+            )
             ct[2]["charset"] = "utf-8"
             self.headers["content-type"] = assemble_content_type(*ct)
             enc = "utf8"
             self.content = text.encode(enc, "surrogateescape")
 
-    def get_text(self, strict: bool = True) -> Optional[str]:
+    def get_text(self, strict: bool = True) -> str | None:
         """
         Similar to `Message.text`, but does not raise if `strict` is `False`.
         Instead, the message body is returned as surrogate-escaped UTF-8.
@@ -476,14 +484,14 @@ class Message(serializable.Serializable):
         self.data.timestamp_start = timestamp_start
 
     @property
-    def timestamp_end(self) -> Optional[float]:
+    def timestamp_end(self) -> float | None:
         """
         *Timestamp:* Last byte received.
         """
         return self.data.timestamp_end
 
     @timestamp_end.setter
-    def timestamp_end(self, timestamp_end: Optional[float]):
+    def timestamp_end(self, timestamp_end: float | None):
         self.data.timestamp_end = timestamp_end
 
     def decode(self, strict: bool = True) -> None:
@@ -510,7 +518,7 @@ class Message(serializable.Serializable):
         self.headers["content-encoding"] = encoding
         self.content = self.raw_content
         if "content-encoding" not in self.headers:
-            raise ValueError("Invalid content encoding {}".format(repr(encoding)))
+            raise ValueError(f"Invalid content encoding {repr(encoding)}")
 
     def json(self, **kwargs: Any) -> Any:
         """
@@ -527,7 +535,7 @@ class Message(serializable.Serializable):
         """
         content = self.get_content(strict=False)
         if content is None:
-            raise TypeError('Message content is not available.')
+            raise TypeError("Message content is not available.")
         else:
             return json.loads(content, **kwargs)
 
@@ -536,6 +544,7 @@ class Request(Message):
     """
     An HTTP request.
     """
+
     data: RequestData
 
     def __init__(
@@ -547,11 +556,11 @@ class Request(Message):
         authority: bytes,
         path: bytes,
         http_version: bytes,
-        headers: Union[Headers, Tuple[Tuple[bytes, bytes], ...]],
-        content: Optional[bytes],
-        trailers: Union[Headers, Tuple[Tuple[bytes, bytes], ...], None],
+        headers: Headers | tuple[tuple[bytes, bytes], ...],
+        content: bytes | None,
+        trailers: Headers | tuple[tuple[bytes, bytes], ...] | None,
         timestamp_start: float,
-        timestamp_end: Optional[float],
+        timestamp_end: float | None,
     ):
         # auto-convert invalid types to retain compatibility with older code.
         if isinstance(host, bytes):
@@ -602,8 +611,10 @@ class Request(Message):
         cls,
         method: str,
         url: str,
-        content: Union[bytes, str] = "",
-        headers: Union[Headers, Dict[Union[str, bytes], Union[str, bytes]], Iterable[Tuple[bytes, bytes]]] = ()
+        content: bytes | str = "",
+        headers: (
+            Headers | dict[str | bytes, str | bytes] | Iterable[tuple[bytes, bytes]]
+        ) = (),
     ) -> "Request":
         """
         Simplified API for creating request objects.
@@ -613,16 +624,20 @@ class Request(Message):
             pass
         elif isinstance(headers, dict):
             headers = Headers(
-                (always_bytes(k, "utf-8", "surrogateescape"),
-                 always_bytes(v, "utf-8", "surrogateescape"))
+                (
+                    always_bytes(k, "utf-8", "surrogateescape"),
+                    always_bytes(v, "utf-8", "surrogateescape"),
+                )
                 for k, v in headers.items()
             )
         elif isinstance(headers, Iterable):
             headers = Headers(headers)  # type: ignore
         else:
-            raise TypeError("Expected headers to be an iterable or dict, but is {}.".format(
-                type(headers).__name__
-            ))
+            raise TypeError(
+                "Expected headers to be an iterable or dict, but is {}.".format(
+                    type(headers).__name__
+                )
+            )
 
         req = cls(
             "",
@@ -646,7 +661,9 @@ class Request(Message):
         elif isinstance(content, str):
             req.text = content
         else:
-            raise TypeError(f"Expected content to be str or bytes, but is {type(content).__name__}.")
+            raise TypeError(
+                f"Expected content to be str or bytes, but is {type(content).__name__}."
+            )
 
         return req
 
@@ -672,7 +689,7 @@ class Request(Message):
         return self.data.method.decode("utf-8", "surrogateescape").upper()
 
     @method.setter
-    def method(self, val: Union[str, bytes]) -> None:
+    def method(self, val: str | bytes) -> None:
         self.data.method = always_bytes(val, "utf-8", "surrogateescape")
 
     @property
@@ -683,7 +700,7 @@ class Request(Message):
         return self.data.scheme.decode("utf-8", "surrogateescape")
 
     @scheme.setter
-    def scheme(self, val: Union[str, bytes]) -> None:
+    def scheme(self, val: str | bytes) -> None:
         self.data.scheme = always_bytes(val, "utf-8", "surrogateescape")
 
     @property
@@ -705,7 +722,7 @@ class Request(Message):
             return self.data.authority.decode("utf8", "surrogateescape")
 
     @authority.setter
-    def authority(self, val: Union[str, bytes]) -> None:
+    def authority(self, val: str | bytes) -> None:
         if isinstance(val, str):
             try:
                 val = val.encode("idna", "strict")
@@ -727,18 +744,12 @@ class Request(Message):
         return self.data.host
 
     @host.setter
-    def host(self, val: Union[str, bytes]) -> None:
+    def host(self, val: str | bytes) -> None:
         self.data.host = always_str(val, "idna", "strict")
-
-        # Update host header
-        if "Host" in self.data.headers:
-            self.data.headers["Host"] = val
-        # Update authority
-        if self.data.authority:
-            self.authority = url.hostport(self.scheme, self.host, self.port)
+        self._update_host_and_authority()
 
     @property
-    def host_header(self) -> Optional[str]:
+    def host_header(self) -> str | None:
         """
         The request's host/authority header.
 
@@ -747,21 +758,21 @@ class Request(Message):
 
         *See also:* `Request.authority`,`Request.host`, `Request.pretty_host`
         """
-        if self.is_http2:
+        if self.is_http2 or self.is_http3:
             return self.authority or self.data.headers.get("Host", None)
         else:
             return self.data.headers.get("Host", None)
 
     @host_header.setter
-    def host_header(self, val: Union[None, str, bytes]) -> None:
+    def host_header(self, val: None | str | bytes) -> None:
         if val is None:
-            if self.is_http2:
+            if self.is_http2 or self.is_http3:
                 self.data.authority = b""
             self.headers.pop("Host", None)
         else:
-            if self.is_http2:
+            if self.is_http2 or self.is_http3:
                 self.authority = val  # type: ignore
-            if not self.is_http2 or "Host" in self.headers:
+            if not (self.is_http2 or self.is_http3) or "Host" in self.headers:
                 # For h2, we only overwrite, but not create, as :authority is the h2 host header.
                 self.headers["Host"] = val
 
@@ -774,7 +785,21 @@ class Request(Message):
 
     @port.setter
     def port(self, port: int) -> None:
+        if not isinstance(port, int):
+            raise ValueError(f"Port must be an integer, not {port!r}.")
+
         self.data.port = port
+        self._update_host_and_authority()
+
+    def _update_host_and_authority(self) -> None:
+        val = url.hostport(self.scheme, self.host, self.port)
+
+        # Update host header
+        if "Host" in self.data.headers:
+            self.data.headers["Host"] = val
+        # Update authority
+        if self.data.authority:
+            self.authority = val
 
     @property
     def path(self) -> str:
@@ -785,7 +810,7 @@ class Request(Message):
         return self.data.path.decode("utf-8", "surrogateescape")
 
     @path.setter
-    def path(self, val: Union[str, bytes]) -> None:
+    def path(self, val: str | bytes) -> None:
         self.data.path = always_bytes(val, "utf-8", "surrogateescape")
 
     @property
@@ -800,7 +825,7 @@ class Request(Message):
         return url.unparse(self.scheme, self.host, self.port, self.path)
 
     @url.setter
-    def url(self, val: Union[str, bytes]) -> None:
+    def url(self, val: str | bytes) -> None:
         val = always_str(val, "utf-8", "surrogateescape")
         self.scheme, self.host, self.port, self.path = url.parse(val)
 
@@ -852,10 +877,7 @@ class Request(Message):
         For the most part, this behaves like a dictionary.
         Modifications to the MultiDictView update `Request.path`, and vice versa.
         """
-        return multidict.MultiDictView(
-            self._get_query,
-            self._set_query
-        )
+        return multidict.MultiDictView(self._get_query, self._set_query)
 
     @query.setter
     def query(self, value):
@@ -875,17 +897,14 @@ class Request(Message):
         For the most part, this behaves like a dictionary.
         Modifications to the MultiDictView update `Request.headers`, and vice versa.
         """
-        return multidict.MultiDictView(
-            self._get_cookies,
-            self._set_cookies
-        )
+        return multidict.MultiDictView(self._get_cookies, self._set_cookies)
 
     @cookies.setter
     def cookies(self, value):
         self._set_cookies(value)
 
     @property
-    def path_components(self) -> Tuple[str, ...]:
+    def path_components(self) -> tuple[str, ...]:
         """
         The URL's path components as a tuple of strings.
         Components are unquoted.
@@ -926,21 +945,22 @@ class Request(Message):
         """
         accept_encoding = self.headers.get("accept-encoding")
         if accept_encoding:
-            self.headers["accept-encoding"] = (
-                ', '.join(
-                    e
-                    for e in {"gzip", "identity", "deflate", "br", "zstd"}
-                    if e in accept_encoding
-                )
+            self.headers["accept-encoding"] = ", ".join(
+                e
+                for e in {"gzip", "identity", "deflate", "br", "zstd"}
+                if e in accept_encoding
             )
 
     def _get_urlencoded_form(self):
-        is_valid_content_type = "application/x-www-form-urlencoded" in self.headers.get("content-type", "").lower()
+        is_valid_content_type = (
+            "application/x-www-form-urlencoded"
+            in self.headers.get("content-type", "").lower()
+        )
         if is_valid_content_type:
             return tuple(url.decode(self.get_text(strict=False)))
         return ()
 
-    def _set_urlencoded_form(self, form_data):
+    def _set_urlencoded_form(self, form_data: Sequence[tuple[str, str]]) -> None:
         """
         Sets the body to the URL-encoded form data, and adds the appropriate content-type header.
         This will overwrite the existing content if there is one.
@@ -959,25 +979,29 @@ class Request(Message):
         Modifications to the MultiDictView update `Request.content`, and vice versa.
         """
         return multidict.MultiDictView(
-            self._get_urlencoded_form,
-            self._set_urlencoded_form
+            self._get_urlencoded_form, self._set_urlencoded_form
         )
 
     @urlencoded_form.setter
     def urlencoded_form(self, value):
         self._set_urlencoded_form(value)
 
-    def _get_multipart_form(self):
-        is_valid_content_type = "multipart/form-data" in self.headers.get("content-type", "").lower()
-        if is_valid_content_type:
+    def _get_multipart_form(self) -> list[tuple[bytes, bytes]]:
+        is_valid_content_type = (
+            "multipart/form-data" in self.headers.get("content-type", "").lower()
+        )
+        if is_valid_content_type and self.content is not None:
             try:
-                return multipart.decode(self.headers.get("content-type"), self.content)
+                return multipart.decode_multipart(
+                    self.headers.get("content-type"), self.content
+                )
             except ValueError:
                 pass
-        return ()
+        return []
 
-    def _set_multipart_form(self, value):
-        is_valid_content_type = self.headers.get("content-type", "").lower().startswith("multipart/form-data")
+    def _set_multipart_form(self, value: list[tuple[bytes, bytes]]) -> None:
+        ct = self.headers.get("content-type", "")
+        is_valid_content_type = ct.lower().startswith("multipart/form-data")
         if not is_valid_content_type:
             """
             Generate a random boundary here.
@@ -986,8 +1010,10 @@ class Request(Message):
             on generating the boundary.
             """
             boundary = "-" * 20 + binascii.hexlify(os.urandom(16)).decode()
-            self.headers["content-type"] = f"multipart/form-data; boundary={boundary}"
-        self.content = multipart.encode(self.headers, value)
+            self.headers[
+                "content-type"
+            ] = ct = f"multipart/form-data; boundary={boundary}"
+        self.content = multipart.encode_multipart(ct, value)
 
     @property
     def multipart_form(self) -> multidict.MultiDictView[bytes, bytes]:
@@ -1000,12 +1026,11 @@ class Request(Message):
         Modifications to the MultiDictView update `Request.content`, and vice versa.
         """
         return multidict.MultiDictView(
-            self._get_multipart_form,
-            self._set_multipart_form
+            self._get_multipart_form, self._set_multipart_form
         )
 
     @multipart_form.setter
-    def multipart_form(self, value):
+    def multipart_form(self, value: list[tuple[bytes, bytes]]) -> None:
         self._set_multipart_form(value)
 
 
@@ -1013,6 +1038,7 @@ class Response(Message):
     """
     An HTTP response.
     """
+
     data: ResponseData
 
     def __init__(
@@ -1020,11 +1046,11 @@ class Response(Message):
         http_version: bytes,
         status_code: int,
         reason: bytes,
-        headers: Union[Headers, Tuple[Tuple[bytes, bytes], ...]],
-        content: Optional[bytes],
-        trailers: Union[None, Headers, Tuple[Tuple[bytes, bytes], ...]],
+        headers: Headers | tuple[tuple[bytes, bytes], ...],
+        content: bytes | None,
+        trailers: None | Headers | tuple[tuple[bytes, bytes], ...],
         timestamp_start: float,
-        timestamp_end: Optional[float],
+        timestamp_end: float | None,
     ):
         # auto-convert invalid types to retain compatibility with older code.
         if isinstance(http_version, str):
@@ -1033,7 +1059,7 @@ class Response(Message):
             reason = reason.encode("ascii", "strict")
 
         if isinstance(content, str):
-            raise ValueError("Content must be bytes, not {}".format(type(content).__name__))
+            raise ValueError(f"Content must be bytes, not {type(content).__name__}")
         if not isinstance(headers, Headers):
             headers = Headers(headers)
         if trailers is not None and not isinstance(trailers, Headers):
@@ -1063,8 +1089,10 @@ class Response(Message):
     def make(
         cls,
         status_code: int = 200,
-        content: Union[bytes, str] = b"",
-        headers: Union[Headers, Mapping[str, Union[str, bytes]], Iterable[Tuple[bytes, bytes]]] = ()
+        content: bytes | str = b"",
+        headers: (
+            Headers | Mapping[str, str | bytes] | Iterable[tuple[bytes, bytes]]
+        ) = (),
     ) -> "Response":
         """
         Simplified API for creating response objects.
@@ -1073,16 +1101,20 @@ class Response(Message):
             headers = headers
         elif isinstance(headers, dict):
             headers = Headers(
-                (always_bytes(k, "utf-8", "surrogateescape"),  # type: ignore
-                 always_bytes(v, "utf-8", "surrogateescape"))
+                (
+                    always_bytes(k, "utf-8", "surrogateescape"),  # type: ignore
+                    always_bytes(v, "utf-8", "surrogateescape"),
+                )
                 for k, v in headers.items()
             )
         elif isinstance(headers, Iterable):
             headers = Headers(headers)  # type: ignore
         else:
-            raise TypeError("Expected headers to be an iterable or dict, but is {}.".format(
-                type(headers).__name__
-            ))
+            raise TypeError(
+                "Expected headers to be an iterable or dict, but is {}.".format(
+                    type(headers).__name__
+                )
+            )
 
         resp = cls(
             b"HTTP/1.1",
@@ -1101,7 +1133,9 @@ class Response(Message):
         elif isinstance(content, str):
             resp.text = content
         else:
-            raise TypeError(f"Expected content to be str or bytes, but is {type(content).__name__}.")
+            raise TypeError(
+                f"Expected content to be str or bytes, but is {type(content).__name__}."
+            )
 
         return resp
 
@@ -1127,16 +1161,13 @@ class Response(Message):
         return self.data.reason.decode("ISO-8859-1")
 
     @reason.setter
-    def reason(self, reason: Union[str, bytes]) -> None:
+    def reason(self, reason: str | bytes) -> None:
         self.data.reason = strutils.always_bytes(reason, "ISO-8859-1")
 
     def _get_cookies(self):
         h = self.headers.get_all("set-cookie")
         all_cookies = cookies.parse_set_cookie_headers(h)
-        return tuple(
-            (name, (value, attrs))
-            for name, value, attrs in all_cookies
-        )
+        return tuple((name, (value, attrs)) for name, value, attrs in all_cookies)
 
     def _set_cookies(self, value):
         cookie_headers = []
@@ -1146,7 +1177,9 @@ class Response(Message):
         self.headers.set_all("set-cookie", cookie_headers)
 
     @property
-    def cookies(self) -> multidict.MultiDictView[str, Tuple[str, multidict.MultiDict[str, Optional[str]]]]:
+    def cookies(
+        self,
+    ) -> multidict.MultiDictView[str, tuple[str, multidict.MultiDict[str, str | None]]]:
         """
         The response cookies. A possibly empty `MultiDictView`, where the keys are cookie
         name strings, and values are `(cookie value, attributes)` tuples. Within
@@ -1156,10 +1189,7 @@ class Response(Message):
         *Warning:* Changes to `attributes` will not be picked up unless you also reassign
         the `(cookie value, attributes)` tuple directly in the `MultiDictView`.
         """
-        return multidict.MultiDictView(
-            self._get_cookies,
-            self._set_cookies
-        )
+        return multidict.MultiDictView(self._get_cookies, self._set_cookies)
 
     @cookies.setter
     def cookies(self, value):
@@ -1206,11 +1236,12 @@ class HTTPFlow(flow.Flow):
     An HTTPFlow is a collection of objects representing a single HTTP
     transaction.
     """
+
     request: Request
     """The client's HTTP request."""
-    response: Optional[Response] = None
+    response: Response | None = None
     """The server's HTTP response."""
-    error: Optional[flow.Error] = None
+    error: flow.Error | None = None
     """
     A connection or protocol error affecting this flow.
 
@@ -1219,27 +1250,37 @@ class HTTPFlow(flow.Flow):
     from the server, but there was an error sending it back to the client.
     """
 
-    websocket: Optional[WebSocketData] = None
+    websocket: WebSocketData | None = None
     """
     If this HTTP flow initiated a WebSocket connection, this attribute contains all associated WebSocket data.
     """
 
-    def __init__(self, client_conn, server_conn, live=None, mode="regular"):
-        super().__init__("http", client_conn, server_conn, live)
-        self.mode = mode
+    def get_state(self) -> serializable.State:
+        return {
+            **super().get_state(),
+            "request": self.request.get_state(),
+            "response": self.response.get_state() if self.response else None,
+            "websocket": self.websocket.get_state() if self.websocket else None,
+        }
 
-    _stateobject_attributes = flow.Flow._stateobject_attributes.copy()
-    # mypy doesn't support update with kwargs
-    _stateobject_attributes.update(dict(
-        request=Request,
-        response=Response,
-        websocket=WebSocketData,
-        mode=str
-    ))
+    def set_state(self, state: serializable.State) -> None:
+        self.request = Request.from_state(state.pop("request"))
+        self.response = Response.from_state(r) if (r := state.pop("response")) else None
+        self.websocket = (
+            WebSocketData.from_state(w) if (w := state.pop("websocket")) else None
+        )
+        super().set_state(state)
 
     def __repr__(self):
         s = "<HTTPFlow"
-        for a in ("request", "response", "websocket", "error", "client_conn", "server_conn"):
+        for a in (
+            "request",
+            "response",
+            "websocket",
+            "error",
+            "client_conn",
+            "server_conn",
+        ):
             if getattr(self, a, False):
                 s += f"\r\n  {a} = {{flow.{a}}}"
         s += ">"
@@ -1250,6 +1291,16 @@ class HTTPFlow(flow.Flow):
         """*Read-only:* An alias for `Request.timestamp_start`."""
         return self.request.timestamp_start
 
+    @property
+    def mode(self) -> str:  # pragma: no cover
+        warnings.warn("HTTPFlow.mode is deprecated.", DeprecationWarning, stacklevel=2)
+        return getattr(self, "_mode", "regular")
+
+    @mode.setter
+    def mode(self, val: str) -> None:  # pragma: no cover
+        warnings.warn("HTTPFlow.mode is deprecated.", DeprecationWarning, stacklevel=2)
+        self._mode = val
+
     def copy(self):
         f = super().copy()
         if self.request:
